@@ -3,11 +3,18 @@ import logging
 import os
 import tempfile
 import sys
+import csv
 from datetime import datetime
 from urllib.parse import urlparse
 import google.generativeai as genai
 from bs4 import BeautifulSoup
 from code_cache import CodeCache
+try:
+    from litellm import completion
+    LITELLM_AVAILABLE = True
+except ImportError:
+    LITELLM_AVAILABLE = False
+    completion = None
 
 class DataExtractor:
     def __init__(self, api_key=None, temp_dir="temp", output_dir="output", model_name=None, enable_cache=True):
@@ -16,6 +23,7 @@ class DataExtractor:
         self.output_dir = output_dir
         self.extraction_codes_dir = os.path.join(temp_dir, "extraction_codes")
         self.enable_cache = enable_cache
+        self.api_key = api_key
         
         # Create directories
         os.makedirs(self.extraction_codes_dir, exist_ok=True)
@@ -31,21 +39,86 @@ class DataExtractor:
             self.code_cache = None
             self.logger.info("Code caching disabled")
         
-        # Configure Gemini API
-        if api_key:
-            genai.configure(api_key=api_key)
-        else:
-            api_key = os.getenv('GEMINI_API_KEY')
-            if not api_key:
-                raise ValueError("Gemini API key not provided. Set GEMINI_API_KEY environment variable or pass api_key parameter.")
-            genai.configure(api_key=api_key)
-        
         # Set model name with default fallback
         self.model_name = model_name or 'gemini-2.5-flash'
-        self.model = genai.GenerativeModel(self.model_name)
+        
+        # Initialize AI provider based on model name
+        self._initialize_ai_provider(api_key)
+        
         self.extraction_history = []
         self.logger.info(f"Initialized DataExtractor with model: {self.model_name}")
     
+    def _initialize_ai_provider(self, api_key):
+        """Initialize AI provider based on model name"""
+        self.use_litellm = False
+        
+        # Check if it's a Gemini model
+        if self.model_name.startswith('gemini'):
+            # Use Google Gemini API directly
+            if api_key:
+                genai.configure(api_key=api_key)
+            else:
+                api_key = os.getenv('GEMINI_API_KEY')
+                if not api_key:
+                    raise ValueError("Gemini API key not provided. Set GEMINI_API_KEY environment variable or pass api_key parameter.")
+                genai.configure(api_key=api_key)
+            
+            self.model = genai.GenerativeModel(self.model_name)
+            self.logger.info(f"Using Google Gemini API with model: {self.model_name}")
+        else:
+            # Use LiteLLM for other providers
+            if not LITELLM_AVAILABLE:
+                raise ImportError("LiteLLM is required for non-Gemini models. Install with: pip install litellm")
+            
+            if not api_key:
+                # For testing purposes, allow initialization without API key
+                self.logger.warning("No API key provided for non-Gemini model - some operations will fail")
+            
+            self.use_litellm = True
+            self.model = None  # LiteLLM doesn't use model objects
+            self.logger.info(f"Using LiteLLM with model: {self.model_name}")
+    
+    def _detect_provider_from_model(self, model_name):
+        """Detect AI provider from model name"""
+        model_name_lower = model_name.lower()
+        
+        if model_name_lower.startswith('gemini'):
+            return 'gemini'
+        elif model_name_lower.startswith('gpt') or 'openai' in model_name_lower:
+            return 'openai'
+        elif model_name_lower.startswith('claude'):
+            return 'anthropic'
+        elif model_name_lower.startswith('llama'):
+            return 'ollama'
+        else:
+            return 'unknown'
+    
+    def _generate_content_with_ai(self, prompt):
+        """Generate content using appropriate AI provider"""
+        if self.use_litellm:
+            # Use LiteLLM for non-Gemini models
+            try:
+                response = completion(
+                    model=self.model_name,
+                    messages=[{"role": "user", "content": prompt}],
+                    api_key=self.api_key
+                )
+                return response.choices[0].message.content
+            except Exception as e:
+                self.logger.error(f"LiteLLM API error: {str(e)}")
+                raise
+        else:
+            # Use Google Gemini API
+            try:
+                response = self.model.generate_content(prompt)
+                if response and response.text:
+                    return response.text
+                else:
+                    raise Exception("No response from Gemini API")
+            except Exception as e:
+                self.logger.error(f"Gemini API error: {str(e)}")
+                raise
+    
     def analyze_html_structure(self, html_content):
         """Analyze HTML to understand the data structure"""
         soup = BeautifulSoup(html_content, 'html.parser')
@@ -152,12 +225,12 @@ Only return the Python code, no explanations.
 """
         
         try:
-            self.logger.info(f"Generating BeautifulSoup code with Gemini for fields: {extraction_fields}")
-            response = self.model.generate_content(prompt)
+            self.logger.info(f"Generating BeautifulSoup code with {self.model_name} for fields: {extraction_fields}")
+            response_text = self._generate_content_with_ai(prompt)
             
-            if response and response.text:
+            if response_text:
                 # Extract Python code from the response
-                code = response.text.strip()
+                code = response_text.strip()
                 
                 # Remove markdown code block markers if present
                 if code.startswith('```python'):
@@ -177,10 +250,10 @@ Only return the Python code, no explanations.
                 self.logger.info("Successfully generated BeautifulSoup code")
                 return code
             else:
-                raise Exception("No response from Gemini API")
+                raise Exception("No response from AI API")
                 
         except Exception as e:
-            self.logger.error(f"Error generating code with Gemini: {str(e)}")
+            self.logger.error(f"Error generating code with AI: {str(e)}")
             raise
     
     def execute_extraction_code(self, code, html_content):
@@ -239,8 +312,15 @@ Only return the Python code, no explanations.
             self.logger.warning(f"Failed to save extraction code: {e}")
             return None
 
-    def save_data(self, data, filename=None, url=None):
-        """Save extracted data to a JSON file in the output directory"""
+    def save_data(self, data, filename=None, url=None, format='json'):
+        """Save extracted data to JSON or CSV file in the output directory
+        
+        Args:
+            data: Extracted data (list of dictionaries or dictionary)
+            filename: Output filename (optional)
+            url: Source URL (optional)
+            format: Output format - 'json' (default) or 'csv'
+        """
         if not filename:
             if url:
                 parsed_url = urlparse(url)
@@ -248,7 +328,8 @@ Only return the Python code, no explanations.
             else:
                 domain = "unknown"
             timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
-            filename = f"{domain}_{timestamp}.json"
+            extension = 'json' if format.lower() == 'json' else 'csv'
+            filename = f"{domain}_{timestamp}.{extension}"
         
         # Ensure filename goes to output directory
         if not os.path.dirname(filename):
@@ -256,19 +337,11 @@ Only return the Python code, no explanations.
         else:
             filepath = filename
         
-        # Prepare the data with metadata
-        output_data = {
-            'extraction_info': {
-                'timestamp': datetime.now().isoformat(),
-                'url': url,
-                'total_items': len(data) if isinstance(data, list) else 1
-            },
-            'data': data
-        }
-        
         try:
-            with open(filepath, 'w', encoding='utf-8') as f:
-                json.dump(output_data, f, indent=2, ensure_ascii=False)
+            if format.lower() == 'csv':
+                self._save_as_csv(data, filepath, url)
+            else:
+                self._save_as_json(data, filepath, url)
             
             self.logger.info(f"Data saved to: {filepath}")
             return filepath
@@ -277,9 +350,62 @@ Only return the Python code, no explanations.
             self.logger.error(f"Error saving data to {filepath}: {str(e)}")
             raise
     
-    def extract_and_save(self, html_content, url=None, output_file=None):
+    def _save_as_json(self, data, filepath, url):
+        """Save data as JSON file"""
+        output_data = {
+            'extraction_info': {
+                'timestamp': datetime.now().isoformat(),
+                'url': url,
+                'total_items': len(data) if isinstance(data, list) else 1,
+                'format': 'json'
+            },
+            'data': data
+        }
+        
+        with open(filepath, 'w', encoding='utf-8') as f:
+            json.dump(output_data, f, indent=2, ensure_ascii=False)
+    
+    def _save_as_csv(self, data, filepath, url):
+        """Save data as CSV file"""
+        if not data:
+            raise ValueError("No data to save to CSV")
+        
+        # Convert single item to list for consistent processing
+        if isinstance(data, dict):
+            data_list = [data]
+        else:
+            data_list = data
+        
+        # Get all unique field names from all items
+        fieldnames = set()
+        for item in data_list:
+            if isinstance(item, dict):
+                fieldnames.update(item.keys())
+        
+        fieldnames = sorted(list(fieldnames))
+        
+        with open(filepath, 'w', newline='', encoding='utf-8') as csvfile:
+            writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
+            
+            # Write header
+            writer.writeheader()
+            
+            # Write data rows
+            for item in data_list:
+                if isinstance(item, dict):
+                    # Ensure all fields are present, fill missing with empty string
+                    row = {field: item.get(field, '') for field in fieldnames}
+                    writer.writerow(row)
+    
+    def extract_and_save(self, html_content, url=None, output_file=None, format='json'):
         """
         Main method to extract data from HTML and save to file
+        
+        Args:
+            html_content: HTML content to extract data from
+            url: Source URL (optional)
+            output_file: Output filename (optional)
+            format: Output format - 'json' (default) or 'csv'
         """
         try:
             self.logger.info("Starting data extraction process...")
@@ -299,7 +425,7 @@ Only return the Python code, no explanations.
             extracted_data = self.execute_extraction_code(extraction_code, html_content)
             
             # Save the data
-            output_filename = self.save_data(extracted_data, output_file, url)
+            output_filename = self.save_data(extracted_data, output_file, url, format)
             
             # Save the generated code to temp folder
             code_filename = self._save_extraction_code(url, extraction_code)
@@ -311,7 +437,8 @@ Only return the Python code, no explanations.
                 'data_file': output_filename,
                 'code_file': code_filename,
                 'extracted_items': len(extracted_data) if isinstance(extracted_data, list) else 1,
-                'extraction_code': extraction_code
+                'extraction_code': extraction_code,
+                'format': format
             }
             
         except Exception as e:
